

import java.net.*;
import java.io.*;


/**
* The object performing all the methods on the
* MotionStar device
*
* @author Stephen Lawrence, modified and scaled down by Joe Starling, Cardiff University
*/
public class MotionStar
{
private final static byte MSG_WAKE_UP = 10;
private final static byte MSG_GET_STATUS = 101;
private final static byte MSG_RUN_CONTINUOUS = 104;
private final static byte MSG_STOP_DATA = 105;
private final static byte MSG_SHUT_DOWN = 11;
private final static byte DATA_PACKET_MULTI = -46;
private final static byte RSP_WAKE_UP = 20;
private final static byte RSP_GET_STATUS = -55;
private final static byte RSP_RUN_CONTINUOUS = -52;
private final static String IP_ADDRESS = "10.74.192.9";
private final static int PORT_NUMBER = 5000;
private InetAddress address;
private DatagramSocket socket;


private int lastMessage;

/**
* Constructor
*
* @see MotionStarException
* @exception thrown if connection error
*/
	public MotionStar() throws MotionStarException {

		try {
			address = InetAddress.getByName(IP_ADDRESS);
		}

		catch (UnknownHostException e)
		{
			throw new MotionStarException("You do not appear to be connected to a network");
		}
		try {

			socket = new DatagramSocket();
			socket.setSoTimeout(300);
		}

		catch (SocketException e) {

			throw new MotionStarException("Cannot connect socket on port " +
						PORT_NUMBER);
		}
}

	
	/**
* Returns an unsigned integer value
* @param i
* @return
* */

private int unsignedInt(int i)
{

	if (i < 0)

		i = 256 + i;
		return i;
	}


/**
* Connect to the MotionStar ̈ device
*
* @see MotionStarException
* @see MocapDevice
* @exception thrown if connection error
*/
public void connect() throws MotionStarException
	{
	sendMessage(MSG_WAKE_UP);
	checkResponse(RSP_WAKE_UP);
	sendMessage(MSG_GET_STATUS);
	checkResponse(RSP_GET_STATUS);
	System.out.println("Succesfully connected to MotionStar");
	}


/**
* Stops the motionstar device from transmitting data
*/
public void stop() throws MotionStarException
	{
	sendMessage(MSG_STOP_DATA);
	System.out.println("*End of data stream.*");
	System.out.println("");
	}


/**
* Disconnects the MotionStar device
*/
public void disconnect() throws MotionStarException 
	{
	stop();
	sendMessage(MSG_SHUT_DOWN);
	}


/**
* Continuously retrieves data from the MotionStar device
*/
public void runContinuous() throws MotionStarException {
	
	sendMessage(MSG_RUN_CONTINUOUS);
	checkResponse(RSP_RUN_CONTINUOUS);
	}

/**
* Sends a message to the motion star device
* @param data A specified byte that is an instruction to the MotionStar device
* @throws MotionStarException
*/
private void sendMessage(byte data) throws MotionStarException
	{
	byte[] message = new byte[16];
	// parts of the datagram correlate to exact parts required
	message[10] = 3;
	message[8] = data;
	lastMessage = data;
	try {
		DatagramPacket packet = new DatagramPacket(message, message.length,
				address, PORT_NUMBER);
				socket.send(packet);
		}
	catch (PortUnreachableException e) 
		{
		throw new MotionStarException("Port " + PORT_NUMBER + " unreachable");
		}
	catch (IOException e) 
	{
		throw new MotionStarException("Cannot communicate with host: " + data);
	}
}


/**
* Receives a message from the motionstar device
* @return The message returned from the device
* @throws MotionStarException
*/
private byte[] receiveMessage() throws MotionStarException {

	boolean connected = false;
	byte[] message = new byte[320];
	int attempt = 0;
	while (attempt < 3)
	{
		attempt++;
		try {
			DatagramPacket packet = new DatagramPacket(message,
					message.length);
			socket.receive(packet);
			connected = true;
			}
		catch (SocketTimeoutException e) {
			System.out.println("Failed connection attempt " + attempt + "/3");
			if (attempt == 3) {

				throw new MotionStarException("Connection timed out: " +
						lastMessage);
			}
			connected = false;
		}

		catch ( IOException exception ) {
			throw new MotionStarException("Connection lost: " + lastMessage);
			}

		if (connected)
			attempt = 4;
		}

	return message;
}


/**
* Checks if the response is as expected / not corrupted
* @param expected@see MocapDevice
* @throws MotionStarException
*/
private void checkResponse(byte expected) throws MotionStarException
{
	byte[] receivedMessage = receiveMessage();
	if (receivedMessage[8] == DATA_PACKET_MULTI) {
		receivedMessage = receiveMessage();
		if (receivedMessage[8] != expected ) {
			throw new MotionStarException("Invalid data received: " +
					unsignedInt(receivedMessage[8]) + "." + unsignedInt(expected));
			}
	}

	else if (receivedMessage[8] != expected ) {
		throw new MotionStarException("Invalid data received: " +
				unsignedInt(receivedMessage[8]) + "." + unsignedInt(expected));
	}
}
}
